@@ -31,12 +31,255 @@
 #include "intel_chipset.h"
 #include "intel_reg.h"
 
+/* INSTDONE */
+# define IDCT_DONE			(1 << 30)
+# define IQ_DONE			(1 << 29)
+# define PR_DONE			(1 << 28)
+# define VLD_DONE			(1 << 27)
+# define IP_DONE			(1 << 26)
+# define FBC_DONE			(1 << 25)
+# define BINNER_DONE			(1 << 24)
+# define SF_DONE			(1 << 23)
+# define SE_DONE			(1 << 22)
+# define WM_DONE			(1 << 21)
+# define IZ_DONE			(1 << 20)
+# define PERSPECTIVE_INTERP_DONE	(1 << 19)
+# define DISPATCHER_DONE		(1 << 18)
+# define PROJECTION_DONE		(1 << 17)
+# define DEPENDENT_ADDRESS_DONE		(1 << 16)
+# define QUAD_CACHE_DONE		(1 << 15)
+# define TEXTURE_FETCH_DONE		(1 << 14)
+# define TEXTURE_DECOMPRESS_DONE	(1 << 13)
+# define SAMPLER_CACHE_DONE		(1 << 12)
+# define FILTER_DONE			(1 << 11)
+# define BYPASS_FIFO_DONE		(1 << 10)
+# define PS_DONE			(1 << 9)
+# define CC_DONE			(1 << 8)
+# define MAP_FILTER_DONE		(1 << 7)
+# define MAP_L2_IDLE			(1 << 6)
+# define RING_2_ENABLE			(1 << 2)
+# define RING_1_ENABLE			(1 << 1)
+# define RING_0_ENABLE			(1 << 0)
+
+# define I830_GMBUS_DONE		(1 << 26)
+# define I830_FBC_DONE			(1 << 25)
+# define I830_BINNER_DONE		(1 << 24)
+# define I830_MPEG_DONE			(1 << 23)
+# define I830_MECO_DONE			(1 << 22)
+# define I830_MCD_DONE			(1 << 21)
+# define I830_MCSTP_DONE		(1 << 20)
+# define I830_CC_DONE			(1 << 19)
+# define I830_DG_DONE			(1 << 18)
+# define I830_DCMP_DONE			(1 << 17)
+# define I830_FTCH_DONE			(1 << 16)
+# define I830_IT_DONE			(1 << 15)
+# define I830_MG_DONE			(1 << 14)
+# define I830_MEC_DONE			(1 << 13)
+# define I830_PC_DONE			(1 << 12)
+# define I830_QCC_DONE			(1 << 11)
+# define I830_TB_DONE			(1 << 10)
+# define I830_WM_DONE			(1 << 9)
+# define I830_EF_DONE			(1 << 8)
+# define I830_BLITTER_DONE		(1 << 7)
+# define I830_MAP_L2_DONE		(1 << 6)
+# define I830_SECONDARY_RING_3_DONE	(1 << 5)
+# define I830_SECONDARY_RING_2_DONE	(1 << 4)
+# define I830_SECONDARY_RING_1_DONE	(1 << 3)
+# define I830_SECONDARY_RING_0_DONE	(1 << 2)
+# define I830_PRIMARY_RING_1_DONE	(1 << 1)
+# define I830_PRIMARY_RING_0_DONE	(1 << 0)
+
+/* INSTDONE_I965 */
+# define I965_ROW_0_EU_0_DONE		(1 << 31)
+# define I965_ROW_0_EU_1_DONE		(1 << 30)
+# define I965_ROW_0_EU_2_DONE		(1 << 29)
+# define I965_ROW_0_EU_3_DONE		(1 << 28)
+# define I965_ROW_1_EU_0_DONE		(1 << 27)
+# define I965_ROW_1_EU_1_DONE		(1 << 26)
+# define I965_ROW_1_EU_2_DONE		(1 << 25)
+# define I965_ROW_1_EU_3_DONE		(1 << 24)
+# define I965_SF_DONE			(1 << 23)
+# define I965_SE_DONE			(1 << 22)
+# define I965_WM_DONE			(1 << 21)
+# define I965_DISPATCHER_DONE		(1 << 18)
+# define I965_PROJECTION_DONE		(1 << 17)
+# define I965_DG_DONE			(1 << 16)
+# define I965_QUAD_CACHE_DONE		(1 << 15)
+# define I965_TEXTURE_FETCH_DONE	(1 << 14)
+# define I965_TEXTURE_DECOMPRESS_DONE	(1 << 13)
+# define I965_SAMPLER_CACHE_DONE	(1 << 12)
+# define I965_FILTER_DONE		(1 << 11)
+# define I965_BYPASS_DONE		(1 << 10)
+# define I965_PS_DONE			(1 << 9)
+# define I965_CC_DONE			(1 << 8)
+# define I965_MAP_FILTER_DONE		(1 << 7)
+# define I965_MAP_L2_IDLE		(1 << 6)
+# define I965_MA_ROW_0_DONE		(1 << 5)
+# define I965_MA_ROW_1_DONE		(1 << 4)
+# define I965_IC_ROW_0_DONE		(1 << 3)
+# define I965_IC_ROW_1_DONE		(1 << 2)
+# define I965_CP_DONE			(1 << 1)
+# define I965_RING_0_ENABLE		(1 << 0)
+
+# define ILK_ROW_0_EU_0_DONE		(1 << 31)
+# define ILK_ROW_0_EU_1_DONE		(1 << 30)
+# define ILK_ROW_0_EU_2_DONE		(1 << 29)
+# define ILK_ROW_0_EU_3_DONE		(1 << 28)
+# define ILK_ROW_1_EU_0_DONE		(1 << 27)
+# define ILK_ROW_1_EU_1_DONE		(1 << 26)
+# define ILK_ROW_1_EU_2_DONE		(1 << 25)
+# define ILK_ROW_1_EU_3_DONE		(1 << 24)
+# define ILK_ROW_2_EU_0_DONE		(1 << 23)
+# define ILK_ROW_2_EU_1_DONE		(1 << 22)
+# define ILK_ROW_2_EU_2_DONE		(1 << 21)
+# define ILK_ROW_2_EU_3_DONE		(1 << 20)
+# define ILK_VCP_DONE			(1 << 19)
+# define ILK_ROW_0_MATH_DONE		(1 << 18)
+# define ILK_ROW_1_MATH_DONE		(1 << 17)
+# define ILK_ROW_2_MATH_DONE		(1 << 16)
+# define ILK_VC1_DONE			(1 << 15)
+# define ILK_ROW_0_MA_DONE		(1 << 14)
+# define ILK_ROW_1_MA_DONE		(1 << 13)
+# define ILK_ROW_2_MA_DONE		(1 << 12)
+# define ILK_ROW_0_ISC_DONE		(1 << 11)
+# define ILK_ROW_1_ISC_DONE		(1 << 10)
+# define ILK_ROW_2_ISC_DONE		(1 << 9)
+# define ILK_VFE_DONE			(1 << 8)
+# define ILK_TD_DONE			(1 << 7)
+# define ILK_SVTS_DONE			(1 << 6)
+# define ILK_TS_DONE			(1 << 5)
+# define ILK_GW_DONE			(1 << 4)
+# define ILK_AI_DONE			(1 << 3)
+# define ILK_AC_DONE			(1 << 2)
+# define ILK_AM_DONE			(1 << 1)
+
+# define GEN6_MA_3_DONE			(1 << 31)
+# define GEN6_EU_32_DONE		(1 << 30)
+# define GEN6_EU_31_DONE		(1 << 29)
+# define GEN6_EU_30_DONE		(1 << 28)
+# define GEN6_MA_2_DONE			(1 << 27)
+# define GEN6_EU_22_DONE		(1 << 26)
+# define GEN6_EU_21_DONE		(1 << 25)
+# define GEN6_EU_20_DONE		(1 << 24)
+# define GEN6_MA_1_DONE			(1 << 23)
+# define GEN6_EU_12_DONE		(1 << 22)
+# define GEN6_EU_11_DONE		(1 << 21)
+# define GEN6_EU_10_DONE		(1 << 20)
+# define GEN6_MA_0_DONE			(1 << 19)
+# define GEN6_EU_02_DONE		(1 << 18)
+# define GEN6_EU_01_DONE		(1 << 17)
+# define GEN6_EU_00_DONE		(1 << 16)
+# define GEN6_IC_3_DONE			(1 << 15)
+# define GEN6_IC_2_DONE			(1 << 14)
+# define GEN6_IC_1_DONE			(1 << 13)
+# define GEN6_IC_0_DONE			(1 << 12)
+# define GEN6_ISC_10_DONE		(1 << 11)
+# define GEN6_ISC_32_DONE		(1 << 10)
+# define GEN6_VSC_DONE			(1 << 9)
+# define GEN6_IEF_DONE			(1 << 8)
+# define GEN6_VFE_DONE			(1 << 7)
+# define GEN6_TD_DONE			(1 << 6)
+# define GEN6_TS_DONE			(1 << 4)
+# define GEN6_GW_DONE			(1 << 3)
+# define GEN6_HIZ_DONE			(1 << 2)
+# define GEN6_AVS_DONE			(1 << 1)
+
+/* INSTDONE_1 */
+# define I965_GW_CS_DONE_CR		(1 << 19)
+# define I965_SVSM_CS_DONE_CR		(1 << 18)
+# define I965_SVDW_CS_DONE_CR		(1 << 17)
+# define I965_SVDR_CS_DONE_CR		(1 << 16)
+# define I965_SVRW_CS_DONE_CR		(1 << 15)
+# define I965_SVRR_CS_DONE_CR		(1 << 14)
+# define I965_SVTW_CS_DONE_CR		(1 << 13)
+# define I965_MASM_CS_DONE_CR		(1 << 12)
+# define I965_MASF_CS_DONE_CR		(1 << 11)
+# define I965_MAW_CS_DONE_CR		(1 << 10)
+# define I965_EM1_CS_DONE_CR		(1 << 9)
+# define I965_EM0_CS_DONE_CR		(1 << 8)
+# define I965_UC1_CS_DONE		(1 << 7)
+# define I965_UC0_CS_DONE		(1 << 6)
+# define I965_URB_CS_DONE		(1 << 5)
+# define I965_ISC_CS_DONE		(1 << 4)
+# define I965_CL_CS_DONE		(1 << 3)
+# define I965_GS_CS_DONE		(1 << 2)
+# define I965_VS0_CS_DONE		(1 << 1)
+# define I965_VF_CS_DONE		(1 << 0)
+
+# define G4X_BCS_DONE			(1 << 31)
+# define G4X_CS_DONE			(1 << 30)
+# define G4X_MASF_DONE			(1 << 29)
+# define G4X_SVDW_DONE			(1 << 28)
+# define G4X_SVDR_DONE			(1 << 27)
+# define G4X_SVRW_DONE			(1 << 26)
+# define G4X_SVRR_DONE			(1 << 25)
+# define G4X_ISC_DONE			(1 << 24)
+# define G4X_MT_DONE			(1 << 23)
+# define G4X_RC_DONE			(1 << 22)
+# define G4X_DAP_DONE			(1 << 21)
+# define G4X_MAWB_DONE			(1 << 20)
+# define G4X_MT_IDLE			(1 << 19)
+# define G4X_GBLT_BUSY			(1 << 18)
+# define G4X_SVSM_DONE			(1 << 17)
+# define G4X_MASM_DONE			(1 << 16)
+# define G4X_QC_DONE			(1 << 15)
+# define G4X_FL_DONE			(1 << 14)
+# define G4X_SC_DONE			(1 << 13)
+# define G4X_DM_DONE			(1 << 12)
+# define G4X_FT_DONE			(1 << 11)
+# define G4X_DG_DONE			(1 << 10)
+# define G4X_SI_DONE			(1 << 9)
+# define G4X_SO_DONE			(1 << 8)
+# define G4X_PL_DONE			(1 << 7)
+# define G4X_WIZ_DONE			(1 << 6)
+# define G4X_URB_DONE			(1 << 5)
+# define G4X_SF_DONE			(1 << 4)
+# define G4X_CL_DONE			(1 << 3)
+# define G4X_GS_DONE			(1 << 2)
+# define G4X_VS0_DONE			(1 << 1)
+# define G4X_VF_DONE			(1 << 0)
+
+/* INSTDONE_1 */
+# define GEN6_GAM_DONE			(1 << 31)
+# define GEN6_CS_DONE			(1 << 30)
+# define GEN6_WMBE_DONE			(1 << 29)
+# define GEN6_SVRW_DONE			(1 << 28)
+# define GEN6_RCC_DONE			(1 << 27)
+# define GEN6_SVG_DONE			(1 << 26)
+# define GEN6_ISC_DONE			(1 << 25)
+# define GEN6_MT_DONE			(1 << 24)
+# define GEN6_RCPFE_DONE		(1 << 23)
+# define GEN6_RCPBE_DONE		(1 << 22)
+# define GEN6_VDI_DONE			(1 << 21)
+# define GEN6_RCZ_DONE			(1 << 20)
+# define GEN6_DAP_DONE			(1 << 19)
+# define GEN6_PSD_DONE			(1 << 18)
+# define GEN6_IZ_DONE			(1 << 17)
+# define GEN6_WMFE_DONE			(1 << 16)
+# define GEN6_SVSM_DONE			(1 << 15)
+# define GEN6_QC_DONE			(1 << 14)
+# define GEN6_FL_DONE			(1 << 13)
+# define GEN6_SC_DONE			(1 << 12)
+# define GEN6_DM_DONE			(1 << 11)
+# define GEN6_FT_DONE			(1 << 10)
+# define GEN6_DG_DONE			(1 << 9)
+# define GEN6_SI_DONE			(1 << 8)
+# define GEN6_SO_DONE			(1 << 7)
+# define GEN6_PL_DONE			(1 << 6)
+# define GEN6_VME_DONE			(1 << 5)
+# define GEN6_SF_DONE			(1 << 4)
+# define GEN6_CL_DONE			(1 << 3)
+# define GEN6_GS_DONE			(1 << 2)
+# define GEN6_VS0_DONE			(1 << 1)
+# define GEN6_VF_DONE			(1 << 0)
+
 struct instdone_bit instdone_bits[MAX_INSTDONE_BITS];
 int num_instdone_bits = 0;
 
 static void
 add_instdone_bit(uint32_t reg, uint32_t bit, const char *name)
 {
+	assert(num_instdone_bits < MAX_INSTDONE_BITS);
 	instdone_bits[num_instdone_bits].reg = reg;
 	instdone_bits[num_instdone_bits].bit = bit;
 	instdone_bits[num_instdone_bits].name = name;
@@ -46,31 +289,31 @@ add_instdone_bit(uint32_t reg, uint32_t bit, const char *name)
 static void
 gen3_instdone_bit(uint32_t bit, const char *name)
 {
-	add_instdone_bit(INST_DONE, bit, name);
+	add_instdone_bit(INSTDONE, bit, name);
 }
 
 static void
 gen4_instdone_bit(uint32_t bit, const char *name)
 {
-	add_instdone_bit(INST_DONE_I965, bit, name);
+	add_instdone_bit(INSTDONE_I965, bit, name);
 }
 
 static void
 gen4_instdone1_bit(uint32_t bit, const char *name)
 {
-	add_instdone_bit(INST_DONE_1, bit, name);
+	add_instdone_bit(INSTDONE_1, bit, name);
 }
 
 static void
 gen6_instdone1_bit(uint32_t bit, const char *name)
 {
-	add_instdone_bit(GEN6_INSTDONE_1, bit, name);
+	add_instdone_bit(INSTDONE_I965, bit, name);
 }
 
 static void
 gen6_instdone2_bit(uint32_t bit, const char *name)
 {
-	add_instdone_bit(GEN6_INSTDONE_2, bit, name);
+	add_instdone_bit(INSTDONE_1, bit, name);
 }
 
 static void
@@ -157,10 +400,28 @@ init_gen7_instdone(void)
 	gen6_instdone1_bit(1 << 1, "VF");
 }
 
+static void
+init_gen75_instdone(void)
+{
+	gen6_instdone1_bit(1 << 21, "CS");
+	gen6_instdone1_bit(1 << 20, "RS");
+	init_gen7_instdone();
+}
+
+static void
+init_gen8_instdone(void)
+{
+	gen6_instdone1_bit(1 << 23, "FBC");
+	gen6_instdone1_bit(1 << 22, "SDE");
+	init_gen75_instdone();
+}
+
 void
 init_instdone_definitions(uint32_t devid)
 {
-	if (IS_GEN7(devid)) {
+	if (IS_GEN8(devid)) {
+		init_gen8_instdone();
+	} else if (IS_GEN7(devid)) {
 		init_gen7_instdone();
 	} else if (IS_GEN6(devid)) {
 		/* Now called INSTDONE_1 in the docs. */
