@@ -47,7 +47,7 @@ void intel_batchbuffer_emit_reloc(struct intel_batchbuffer *batch,
  * work...
  */
 #pragma GCC diagnostic ignored "-Winline"
-static inline int
+static inline unsigned int
 intel_batchbuffer_space(struct intel_batchbuffer *batch)
 {
 	return (BATCH_SZ - BATCH_RESERVED) - (batch->ptr - batch->buffer);
@@ -96,6 +96,43 @@ intel_batchbuffer_require_space(struct intel_batchbuffer *batch,
 #define ADVANCE_BATCH() do {						\
 } while(0)
 
+#define BLIT_COPY_BATCH_START(devid, flags) do { \
+	if (intel_gen(devid) >= 8) { \
+		BEGIN_BATCH(10); \
+		OUT_BATCH(XY_SRC_COPY_BLT_CMD | \
+				XY_SRC_COPY_BLT_WRITE_ALPHA | \
+				XY_SRC_COPY_BLT_WRITE_RGB | \
+				(flags) | 8); \
+	} else { \
+		BEGIN_BATCH(8); \
+		OUT_BATCH(XY_SRC_COPY_BLT_CMD | \
+				XY_SRC_COPY_BLT_WRITE_ALPHA | \
+				XY_SRC_COPY_BLT_WRITE_RGB | \
+				(flags) | 6); \
+	} \
+} while(0)
+
+#define COLOR_BLIT_COPY_BATCH_START(devid, flags) do { \
+	if (intel_gen(devid) >= 8) { \
+		BEGIN_BATCH(8); \
+		OUT_BATCH(MI_NOOP); \
+		OUT_BATCH(XY_COLOR_BLT_CMD_NOLEN | 0x5 | \
+				COLOR_BLT_WRITE_ALPHA | \
+				XY_COLOR_BLT_WRITE_RGB); \
+	} else { \
+		BEGIN_BATCH(6); \
+		OUT_BATCH(XY_COLOR_BLT_CMD_NOLEN | 0x4 | \
+				COLOR_BLT_WRITE_ALPHA | \
+				XY_COLOR_BLT_WRITE_RGB); \
+	} \
+} while(0)
+
+#define BLIT_RELOC_UDW(devid) do { \
+	if (intel_gen(devid) >= 8) { \
+		OUT_BATCH(0); \
+	} \
+} while(0)
+
 void
 intel_blt_copy(struct intel_batchbuffer *batch,
 	      drm_intel_bo *src_bo, int src_x1, int src_y1, int src_pitch,
