@@ -43,6 +43,8 @@
 
 #include "intel_gpu_tools.h"
 
+#define FAKEKEY 0x2468ace0
+
 void *mmio;
 
 static struct _mmio_data {
@@ -92,7 +94,7 @@ intel_get_mmio(struct pci_device *pci_dev)
 
 	gen = intel_gen(devid);
 	if (gen < 3)
-		mmio_size = 64*1024;
+		mmio_size = 512*1024;
 	else if (gen < 5)
 		mmio_size = 512*1024;
 	else
@@ -173,35 +175,44 @@ intel_register_access_init(struct pci_device *pci_dev, int safe)
 	if (mmio_data.inited)
 		return -1;
 
-	mmio_data.safe = safe != 0 ? true : false;
+	mmio_data.safe = (safe != 0 &&
+			intel_gen(pci_dev->device_id) >= 4) ? true : false;
 	mmio_data.i915_devid = pci_dev->device_id;
 	if (mmio_data.safe)
 		mmio_data.map = intel_get_register_map(mmio_data.i915_devid);
 
-	if (!(IS_GEN6(pci_dev->device_id) ||
-	      IS_GEN7(pci_dev->device_id)))
-		goto done;
-
-	/* Find where the forcewake lock is */
+	/* Find where the forcewake lock is. Forcewake doesn't exist
+	 * gen < 6, but the debugfs should do the right things for us.
+	 */
 	ret = find_debugfs_path("/sys/kernel/debug/dri");
 	if (ret) {
 		ret = find_debugfs_path("/debug/dri");
 		if (ret) {
 			fprintf(stderr, "Couldn't find path to dri/debugfs entry\n");
-			return ret;
+			fprintf(stderr, "warning: forcewake will not be handled\n");
 		}
-	}
-	mmio_data.key = get_forcewake_lock();
+		mmio_data.key = FAKEKEY;
+	} else
+		mmio_data.key = get_forcewake_lock();
 
-done:
 	mmio_data.inited++;
 	return 0;
 }
+static int
+intel_register_access_needs_wake(void)
+{
+	return mmio_data.key != FAKEKEY;
+}
+
+int intel_register_access_needs_fakewake(void)
+{
+	return mmio_data.key == FAKEKEY;
+}
 
 void
 intel_register_access_fini(void)
 {
-	if (mmio_data.key)
+	if (mmio_data.key && intel_register_access_needs_wake())
 		release_forcewake_lock(mmio_data.key);
 	mmio_data.inited--;
 }
