@@ -1,3 +1,6 @@
+#ifndef RENDERCOPY_H
+#define RENDERCOPY_H
+
 #include <stdlib.h>
 #include <sys/ioctl.h>
 #include <stdio.h>
@@ -59,23 +62,37 @@ static inline unsigned buf_height(struct scratch_buf *buf)
 }
 
 typedef void (*render_copyfunc_t)(struct intel_batchbuffer *batch,
+				  drm_intel_context *context,
 				  struct scratch_buf *src, unsigned src_x, unsigned src_y,
 				  unsigned width, unsigned height,
 				  struct scratch_buf *dst, unsigned dst_x, unsigned dst_y);
 
+render_copyfunc_t get_render_copyfunc(int devid);
+
+void gen8_render_copyfunc(struct intel_batchbuffer *batch,
+			  drm_intel_context *context,
+			  struct scratch_buf *src, unsigned src_x, unsigned src_y,
+			  unsigned width, unsigned height,
+			  struct scratch_buf *dst, unsigned dst_x, unsigned dst_y);
 void gen7_render_copyfunc(struct intel_batchbuffer *batch,
+			  drm_intel_context *context,
 			  struct scratch_buf *src, unsigned src_x, unsigned src_y,
 			  unsigned width, unsigned height,
 			  struct scratch_buf *dst, unsigned dst_x, unsigned dst_y);
 void gen6_render_copyfunc(struct intel_batchbuffer *batch,
+			  drm_intel_context *context,
 			  struct scratch_buf *src, unsigned src_x, unsigned src_y,
 			  unsigned width, unsigned height,
 			  struct scratch_buf *dst, unsigned dst_x, unsigned dst_y);
 void gen3_render_copyfunc(struct intel_batchbuffer *batch,
+			  drm_intel_context *context,
 			  struct scratch_buf *src, unsigned src_x, unsigned src_y,
 			  unsigned width, unsigned height,
 			  struct scratch_buf *dst, unsigned dst_x, unsigned dst_y);
 void gen2_render_copyfunc(struct intel_batchbuffer *batch,
+			  drm_intel_context *context,
 			  struct scratch_buf *src, unsigned src_x, unsigned src_y,
 			  unsigned width, unsigned height,
 			  struct scratch_buf *dst, unsigned dst_x, unsigned dst_y);
+
+#endif /* RENDERCOPY_H */
