@@ -31,11 +31,13 @@
  * Historically the batch prefetcher doesn't check whether it's crossing page
  * boundaries and likes to throw up when it gets a pagefault in return for his
  * over-eager behaviour. Check for this.
+ *
+ * This test for a bug where we've failed to plug a scratch pte entry into the
+ * very last gtt pte.
  */
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-#include <assert.h>
 #include <fcntl.h>
 #include <inttypes.h>
 #include <errno.h>
@@ -82,10 +84,10 @@ static void exec(int fd, uint32_t handle)
 		       DRM_IOCTL_I915_GEM_EXECBUFFER2,
 		       &execbuf);
 	gem_sync(fd, handle);
-	assert(ret == 0);
+	igt_assert(ret == 0);
 }
 
-int main(int argc, char **argv)
+igt_simple_main
 {
 	uint32_t batch_end[4] = {MI_BATCH_BUFFER_END, 0, 0, 0};
 	int fd, i, ret;
@@ -93,57 +95,53 @@ int main(int argc, char **argv)
 	int count;
 	drm_intel_bo *sample_batch_bo;
 
+	igt_skip_on_simulation();
+
 	fd = drm_open_any();
 
 	bufmgr = drm_intel_bufmgr_gem_init(fd, 4096);
-	if (!bufmgr) {
-		fprintf(stderr, "failed to init libdrm\n");
-		exit(-1);
-	}
+	igt_assert(bufmgr);
 
 	drm_intel_bufmgr_gem_enable_reuse(bufmgr);
 	
 	aper_size = gem_aperture_size(fd);
 
 	/* presume a big per-bo overhead */
-	if (intel_get_total_ram_mb() < (aper_size / (1024*1024)) * 3 / 2) {
-		fprintf(stderr, "not enough mem to run test\n");
-		return 77;
-	}
+	igt_skip_on_f(intel_get_total_ram_mb() < (aper_size / (1024*1024)) * 3 / 2,
+		      "not enough mem to run test\n");
 
 	count = aper_size / 4096;
 
 	batch = intel_batchbuffer_alloc(bufmgr, intel_get_drm_devid(fd));
-	assert(batch);
+	igt_assert(batch);
 
 	sample_batch_bo = drm_intel_bo_alloc(bufmgr, "", 4096, 4096);
-	assert(sample_batch_bo);
+	igt_assert(sample_batch_bo);
 	ret = drm_intel_bo_subdata(sample_batch_bo, 4096-sizeof(batch_end),
 				   sizeof(batch_end), batch_end);
-	assert(ret == 0);
+	igt_assert(ret == 0);
 
 	/* fill the entire gart with batches and run them */
 	for (i = 0; i < count; i++) {
 		drm_intel_bo *batch_bo;
 
 		batch_bo = drm_intel_bo_alloc(bufmgr, "", 4096, 4096);
-		assert(batch_bo);
+		igt_assert(batch_bo);
 
 		/* copy the sample batch with the gpu to the new one, so that we
 		 * also test the unmappable part of the gtt. */
-		BEGIN_BATCH(8);
-		OUT_BATCH(XY_SRC_COPY_BLT_CMD |
-			  XY_SRC_COPY_BLT_WRITE_ALPHA |
-			  XY_SRC_COPY_BLT_WRITE_RGB);
+		BLIT_COPY_BATCH_START(batch->devid, 0);
 		OUT_BATCH((3 << 24) | /* 32 bits */
 			  (0xcc << 16) | /* copy ROP */
 			  4096);
 		OUT_BATCH(0); /* dst y1,x1 */
 		OUT_BATCH((1 << 16) | 1024);
 		OUT_RELOC(batch_bo, I915_GEM_DOMAIN_RENDER, I915_GEM_DOMAIN_RENDER, 0);
+		BLIT_RELOC_UDW(batch->devid);
 		OUT_BATCH((0 << 16) | 0); /* src x1, y1 */
 		OUT_BATCH(4096);
 		OUT_RELOC(sample_batch_bo, I915_GEM_DOMAIN_RENDER, 0, 0);
+		BLIT_RELOC_UDW(batch->devid);
 		ADVANCE_BATCH();
 
 		intel_batchbuffer_flush(batch);
@@ -157,13 +155,11 @@ int main(int argc, char **argv)
 
 		// leak buffers
 		//drm_intel_bo_unreference(batch_bo);
-		drmtest_progress("gem_cs_prefetch: ", i, count);
+		igt_progress("gem_cs_prefetch: ", i, count);
 	}
 
-	fprintf(stderr, "Test suceeded, cleanup up - this might take a while.\n");
+	printf("Test suceeded, cleanup up - this might take a while.\n");
 	drm_intel_bufmgr_destroy(bufmgr);
 
 	close(fd);
-
-	return 0;
 }
