@@ -31,13 +31,11 @@
 #include <stdint.h>
 #include <stdio.h>
 #include <string.h>
-#include <assert.h>
 #include <fcntl.h>
 #include <inttypes.h>
 #include <errno.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
-#include <sys/mman.h>
 #include <sys/time.h>
 #include "drm.h"
 #include "i915_drm.h"
@@ -61,6 +59,10 @@ int main(int argc, char **argv)
 	int loop, i, tiling;
 	int fd;
 
+	igt_simple_init();
+
+	igt_skip_on_simulation();
+
 	if (argc > 1)
 		size = atoi(argv[1]);
 	if (size == 0) {
@@ -73,7 +75,7 @@ int main(int argc, char **argv)
 	fd = drm_open_any();
 
 	handle = gem_create(fd, size);
-	assert(handle);
+	igt_assert(handle);
 
 	for (tiling = I915_TILING_NONE; tiling <= I915_TILING_Y; tiling++) {
 		if (tiling != I915_TILING_NONE) {
@@ -133,6 +135,25 @@ int main(int argc, char **argv)
 				gettimeofday(&end, NULL);
 				printf("Time to write %dk through a CPU map:		%7.3fµs\n",
 				       size/1024, elapsed(&start, &end, loop));
+
+				gettimeofday(&start, NULL);
+				for (loop = 0; loop < 1000; loop++) {
+					base = gem_mmap__cpu(fd, handle, size, PROT_READ | PROT_WRITE);
+					memset(base, 0, size);
+					munmap(base, size);
+				}
+				gettimeofday(&end, NULL);
+				printf("Time to clear %dk through a CPU map:		%7.3fµs\n",
+				       size/1024, elapsed(&start, &end, loop));
+
+				gettimeofday(&start, NULL);
+				base = gem_mmap__cpu(fd, handle, size, PROT_READ | PROT_WRITE);
+				for (loop = 0; loop < 1000; loop++)
+					memset(base, 0, size);
+				munmap(base, size);
+				gettimeofday(&end, NULL);
+				printf("Time to clear %dk through a cached CPU map:	%7.3fµs\n",
+				       size/1024, elapsed(&start, &end, loop));
 			}
 
 			/* CPU pwrite */
@@ -200,6 +221,26 @@ int main(int argc, char **argv)
 		printf("Time to write %dk through a GTT map:		%7.3fµs\n",
 		       size/1024, elapsed(&start, &end, loop));
 
+		/* mmap clear */
+		gettimeofday(&start, NULL);
+		for (loop = 0; loop < 1000; loop++) {
+			uint32_t *base = gem_mmap(fd, handle, size, PROT_READ | PROT_WRITE);
+			memset(base, 0, size);
+			munmap(base, size);
+		}
+		gettimeofday(&end, NULL);
+		printf("Time to clear %dk through a GTT map:		%7.3fµs\n",
+		       size/1024, elapsed(&start, &end, loop));
+
+		gettimeofday(&start, NULL);{
+			uint32_t *base = gem_mmap(fd, handle, size, PROT_READ | PROT_WRITE);
+			for (loop = 0; loop < 1000; loop++)
+				memset(base, 0, size);
+			munmap(base, size);
+		} gettimeofday(&end, NULL);
+		printf("Time to clear %dk through a cached GTT map:	%7.3fµs\n",
+		       size/1024, elapsed(&start, &end, loop));
+
 		/* mmap read */
 		gettimeofday(&start, NULL);
 		for (loop = 0; loop < 1000; loop++) {
