@@ -118,7 +118,7 @@ struct {
 
 static void tile2xy(struct scratch_buf *buf, unsigned tile, unsigned *x, unsigned *y)
 {
-	assert(tile < buf->num_tiles);
+	igt_assert(tile < buf->num_tiles);
 	*x = (tile*options.tile_size) % (buf->stride/sizeof(uint32_t));
 	*y = ((tile*options.tile_size) / (buf->stride/sizeof(uint32_t))) * options.tile_size;
 }
@@ -141,20 +141,18 @@ static void emit_blt(drm_intel_bo *src_bo, uint32_t src_tiling, unsigned src_pit
 	}
 
 	/* copy lower half to upper half */
-	BEGIN_BATCH(8);
-	OUT_BATCH(XY_SRC_COPY_BLT_CMD |
-		  XY_SRC_COPY_BLT_WRITE_ALPHA |
-		  XY_SRC_COPY_BLT_WRITE_RGB |
-		  cmd_bits);
+	BLIT_COPY_BATCH_START(devid, cmd_bits);
 	OUT_BATCH((3 << 24) | /* 32 bits */
 		  (0xcc << 16) | /* copy ROP */
 		  dst_pitch);
 	OUT_BATCH(dst_y << 16 | dst_x);
 	OUT_BATCH((dst_y+h) << 16 | (dst_x+w));
 	OUT_RELOC_FENCED(dst_bo, I915_GEM_DOMAIN_RENDER, I915_GEM_DOMAIN_RENDER, 0);
+	BLIT_RELOC_UDW(devid);
 	OUT_BATCH(src_y << 16 | src_x);
 	OUT_BATCH(src_pitch);
 	OUT_RELOC_FENCED(src_bo, I915_GEM_DOMAIN_RENDER, 0, 0);
+	BLIT_RELOC_UDW(devid);
 	ADVANCE_BATCH();
 
 	if (IS_GEN6(devid) || IS_GEN7(devid)) {
@@ -173,7 +171,7 @@ static void keep_gpu_busy(void)
 	int tmp;
 
 	tmp = 1 << gpu_busy_load;
-	assert(tmp <= 1024);
+	igt_assert(tmp <= 1024);
 
 	emit_blt(busy_bo, 0, 4096, 0, 0, tmp, 128,
 		 busy_bo, 0, 4096, 0, 128);
@@ -209,7 +207,7 @@ static void cpucpy2d(uint32_t *src, unsigned src_stride, unsigned src_x, unsigne
 			    printf("mismatch at tile %i pos %i, read %i, expected %i, diff %i\n",
 				    logical_tile_no, i*options.tile_size + j, tmp, expect, (int) tmp - expect);
 			    if (options.trace_tile >= 0 && options.fail)
-				    exit(1);
+				    igt_fail(1);
 			    failed++;
 			}
 			/* when not aborting, correct any errors */
@@ -217,7 +215,7 @@ static void cpucpy2d(uint32_t *src, unsigned src_stride, unsigned src_x, unsigne
 		}
 	}
 	if (failed && options.fail)
-		exit(1);
+		igt_fail(1);
 
 	if (failed > stats.max_failed_reads)
 		stats.max_failed_reads = failed;
@@ -229,7 +227,7 @@ static void cpu_copyfunc(struct scratch_buf *src, unsigned src_x, unsigned src_y
 			 struct scratch_buf *dst, unsigned dst_x, unsigned dst_y,
 			 unsigned logical_tile_no)
 {
-	assert(batch->ptr == batch->buffer);
+	igt_assert(batch->ptr == batch->buffer);
 
 	if (options.ducttape)
 		drm_intel_bo_wait_rendering(dst->bo);
@@ -251,7 +249,7 @@ static void prw_copyfunc(struct scratch_buf *src, unsigned src_x, unsigned src_y
 	uint32_t tmp_tile[options.tile_size*options.tile_size];
 	int i;
 
-	assert(batch->ptr == batch->buffer);
+	igt_assert(batch->ptr == batch->buffer);
 
 	if (options.ducttape)
 		drm_intel_bo_wait_rendering(dst->bo);
@@ -323,31 +321,16 @@ static void render_copyfunc(struct scratch_buf *src, unsigned src_x, unsigned sr
 			    unsigned logical_tile_no)
 {
 	static unsigned keep_gpu_busy_counter = 0;
+	render_copyfunc_t rendercopy = get_render_copyfunc(devid);
 
 	/* check both edges of the fence usage */
 	if (keep_gpu_busy_counter & 1)
 		keep_gpu_busy();
 
-	if (IS_GEN2(devid))
-		gen2_render_copyfunc(batch,
-				     src, src_x, src_y,
-				     options.tile_size, options.tile_size,
-				     dst, dst_x, dst_y);
-	else if (IS_GEN3(devid))
-		gen3_render_copyfunc(batch,
-				     src, src_x, src_y,
-				     options.tile_size, options.tile_size,
-				     dst, dst_x, dst_y);
-	else if (IS_GEN6(devid))
-		gen6_render_copyfunc(batch,
-				     src, src_x, src_y,
-				     options.tile_size, options.tile_size,
-				     dst, dst_x, dst_y);
-	else if (IS_GEN7(devid))
-		gen7_render_copyfunc(batch,
-				     src, src_x, src_y,
-				     options.tile_size, options.tile_size,
-				     dst, dst_x, dst_y);
+	if (rendercopy)
+		rendercopy(batch, NULL, src, src_x, src_y,
+		     options.tile_size, options.tile_size,
+		     dst, dst_x, dst_y);
 	else
 		blitter_copyfunc(src, src_x, src_y,
 				 dst, dst_x, dst_y,
@@ -465,12 +448,12 @@ static void sanitize_stride(struct scratch_buf *buf)
 	if (buf_width(buf) < options.tile_size)
 		buf->stride = options.tile_size * sizeof(uint32_t);
 
-	assert(buf->stride <= 8192);
-	assert(buf_width(buf) <= options.max_dimension);
-	assert(buf_height(buf) <= options.max_dimension);
+	igt_assert(buf->stride <= 8192);
+	igt_assert(buf_width(buf) <= options.max_dimension);
+	igt_assert(buf_height(buf) <= options.max_dimension);
 
-	assert(buf_width(buf) >= options.tile_size);
-	assert(buf_height(buf) >= options.tile_size);
+	igt_assert(buf_width(buf) >= options.tile_size);
+	igt_assert(buf_height(buf) >= options.tile_size);
 
 }
 
@@ -478,7 +461,7 @@ static void init_buffer(struct scratch_buf *buf, unsigned size)
 {
 	buf->bo = drm_intel_bo_alloc(bufmgr, "tiled bo", size, 4096);
 	buf->size = size;
-	assert(buf->bo);
+	igt_assert(buf->bo);
 	buf->tiling = I915_TILING_NONE;
 	buf->stride = 4096;
 
@@ -513,7 +496,7 @@ static void init_set(unsigned set)
 	long int r;
 	int i;
 
-	drmtest_permute_array(buffers[set], num_buffers, exchange_buf);
+	igt_permute_array(buffers[set], num_buffers, exchange_buf);
 
 	if (current_set == 1 && options.gpu_busy_load == 0) {
 		gpu_busy_load++;
@@ -618,22 +601,6 @@ static void copy_tiles(unsigned *permutation)
 	intel_batchbuffer_flush(batch);
 }
 
-static int get_num_fences(void)
-{
-	drm_i915_getparam_t gp;
-	int ret, val;
-
-	gp.param = I915_PARAM_NUM_FENCES_AVAIL;
-	gp.value = &val;
-	ret = drmIoctl(drm_fd, DRM_IOCTL_I915_GETPARAM, &gp);
-	assert (ret == 0);
-
-	printf ("total %d fences\n", val);
-	assert(val > 4);
-
-	return val - 2;
-}
-
 static void sanitize_tiles_per_buf(void)
 {
 	if (options.tiles_per_buf > options.scratch_buf_size / TILE_BYTES(options.tile_size))
@@ -665,6 +632,7 @@ static void parse_options(int argc, char **argv)
 		{"tile-size", 1, 0, TILESZ},
 #define CHCK_RENDER 0xdead0003
 		{"check-render-cpyfn", 0, 0, CHCK_RENDER},
+		{NULL, 0, 0, 0},
 	};
 
 	options.scratch_buf_size = 256*4096;
@@ -813,7 +781,8 @@ static void init(void)
 	bufmgr = drm_intel_bufmgr_gem_init(drm_fd, 4096);
 	drm_intel_bufmgr_gem_enable_reuse(bufmgr);
 	drm_intel_bufmgr_gem_enable_fenced_relocs(bufmgr);
-	num_fences = get_num_fences();
+	num_fences = gem_available_fences(drm_fd);
+	igt_assert(num_fences > 4);
 	batch = intel_batchbuffer_alloc(bufmgr, devid);
 
 	busy_bo = drm_intel_bo_alloc(bufmgr, "tiled bo", BUSY_BUF_SIZE, 4096);
@@ -889,7 +858,7 @@ int main(int argc, char **argv)
 
 	/* start our little helper early before too may allocations occur */
 	if (options.use_signal_helper)
-		drmtest_fork_signal_helper();
+		igt_fork_signal_helper();
 
 	init();
 
@@ -898,9 +867,9 @@ int main(int argc, char **argv)
 	tile_permutation = malloc(num_total_tiles*sizeof(uint32_t));
 	current_permutation = malloc(num_total_tiles*sizeof(uint32_t));
 	tmp_permutation = malloc(num_total_tiles*sizeof(uint32_t));
-	assert(tile_permutation);
-	assert(current_permutation);
-	assert(tmp_permutation);
+	igt_assert(tile_permutation);
+	igt_assert(current_permutation);
+	igt_assert(tmp_permutation);
 
 	fan_out();
 
@@ -916,7 +885,7 @@ int main(int argc, char **argv)
 
 		for (j = 0; j < num_total_tiles; j++)
 			current_permutation[j] = j;
-		drmtest_permute_array(current_permutation, num_total_tiles, exchange_uint);
+		igt_permute_array(current_permutation, num_total_tiles, exchange_uint);
 
 		copy_tiles(current_permutation);
 
@@ -939,7 +908,7 @@ int main(int argc, char **argv)
 
 	close(drm_fd);
 
-	drmtest_stop_signal_helper();
+	igt_stop_signal_helper();
 
 	return 0;
 }
