@@ -125,6 +125,8 @@ typedef enum {
 #define SPD_INFOFRAME_VERSION 0x01
 #define SPD_INFOFRAME_LENGTH  0x19
 
+#define VENDOR_ID_HDMI	0x000c03
+
 typedef struct {
 	uint8_t type;
 	uint8_t version;
@@ -173,6 +175,26 @@ typedef union {
 		uint8_t description[16];
 		uint8_t source;
 	} __attribute__((packed)) spd;
+	struct {
+		DipInfoFrameHeader header;
+		uint8_t checksum;
+
+		uint8_t id[3];
+
+		uint8_t Rsvd0        :5;
+		uint8_t video_format :3;
+
+		union {
+			uint8_t vic;
+			struct {
+				uint8_t Rsvd1         :4;
+				uint8_t s3d_structure :4;
+			} s3d;
+		} pb5;
+
+		uint8_t Rsvd2        :4;
+		uint8_t s3d_ext_data :4;
+	} __attribute__((packed)) vendor;
 	struct {
 		DipInfoFrameHeader header;
 		uint8_t body[27];
@@ -381,7 +403,7 @@ static void dump_raw_infoframe(DipInfoFrame *frame)
 static void dump_avi_info(Transcoder transcoder)
 {
 	Register reg = get_dip_ctl_reg(transcoder);
-	uint32_t val = INREG(reg);
+	uint32_t val;
 	DipFrequency freq;
 	DipInfoFrame frame;
 
@@ -411,7 +433,7 @@ static void dump_avi_info(Transcoder transcoder)
 	       frame.avi.R, frame.avi.M, frame.avi.C);
 	printf("- SC: %x, Q: %x, EC: %x, ITC: %x\n",
 	       frame.avi.SC, frame.avi.Q, frame.avi.EC, frame.avi.ITC);
-	printf("- VIC: %x, Rsvd1: %x\n", frame.avi.VIC, frame.avi.Rsvd1);
+	printf("- VIC: %d, Rsvd1: %x\n", frame.avi.VIC, frame.avi.Rsvd1);
 	printf("- PR: %x, Rsvd2: %x\n", frame.avi.PR, frame.avi.Rsvd2);
 	printf("- top: %x, bottom: %x, left: %x, right: %x\n",
 	       frame.avi.top, frame.avi.bottom, frame.avi.left,
@@ -424,10 +446,63 @@ static void dump_avi_info(Transcoder transcoder)
 		printf("Invalid InfoFrame checksum!\n");
 }
 
+static const char *vendor_id_to_string(uint32_t id)
+{
+	switch (id) {
+	case VENDOR_ID_HDMI:
+		return "HDMI";
+	default:
+		return "Unknown";
+	}
+}
+
+static const char *s3d_structure_to_string(int format)
+{
+	switch (format) {
+	case 0:
+		return "Frame Packing";
+	case 6:
+		return "Top Bottom";
+	case 8:
+		return "Side By Side (half)";
+	default:
+		return "Reserved";
+	}
+}
+
+static void dump_vendor_hdmi(DipInfoFrame *frame)
+{
+	int vic_present = frame->vendor.video_format & 0x1;
+	int s3d_present = frame->vendor.video_format & 0x2;
+
+	printf("- video format: 0x%03x %s\n", frame->vendor.video_format,
+	       s3d_present ? "(3D)" : "");
+
+	if (vic_present && s3d_present) {
+		printf("Error: HDMI VIC and S3D bits set. Only one of those "
+		       " at a time is valid\n");
+		return;
+	}
+
+	if (vic_present)
+		printf("- HDMI VIC: %d\n", frame->vendor.pb5.vic);
+	else if (s3d_present) {
+		int s3d_structure = frame->vendor.pb5.s3d.s3d_structure;
+
+		printf("- 3D Format: %s\n",
+		       s3d_structure_to_string(s3d_structure));
+
+		/* Side-by-side (half) */
+		if (s3d_structure >= 8)
+			printf("- 3D Ext Data 0x%x\n",
+			       frame->vendor.s3d_ext_data);
+	}
+}
+
 static void dump_vendor_info(Transcoder transcoder)
 {
 	Register reg = get_dip_ctl_reg(transcoder);
-	uint32_t val = INREG(reg);
+	uint32_t val, vendor_id;
 	DipFrequency freq;
 	DipInfoFrame frame;
 
@@ -446,6 +521,15 @@ static void dump_vendor_info(Transcoder transcoder)
 
 	dump_raw_infoframe(&frame);
 
+	vendor_id = frame.vendor.id[2] << 16 | frame.vendor.id[1] << 8 |
+		    frame.vendor.id[0];
+
+	printf("- vendor Id: 0x%06x (%s)\n", vendor_id,
+	       vendor_id_to_string(vendor_id));
+
+	if (vendor_id == VENDOR_ID_HDMI)
+		dump_vendor_hdmi(&frame);
+
 	if (!infoframe_valid_checksum(&frame))
 		printf("Invalid InfoFrame checksum!\n");
 }
@@ -453,7 +537,7 @@ static void dump_vendor_info(Transcoder transcoder)
 static void dump_gamut_info(Transcoder transcoder)
 {
 	Register reg = get_dip_ctl_reg(transcoder);
-	uint32_t val = INREG(reg);
+	uint32_t val;
 	DipFrequency freq;
 	DipInfoFrame frame;
 
@@ -479,7 +563,7 @@ static void dump_gamut_info(Transcoder transcoder)
 static void dump_spd_info(Transcoder transcoder)
 {
 	Register reg = get_dip_ctl_reg(transcoder);
-	uint32_t val = INREG(reg);
+	uint32_t val;
 	DipFrequency freq;
 	DipInfoFrame frame;
 	char vendor[9];
@@ -660,7 +744,7 @@ static int parse_infoframe_option_s(const char *name, const char *s,
 static void change_avi_infoframe(Transcoder transcoder, char *commands)
 {
 	Register reg = get_dip_ctl_reg(transcoder);
-	uint32_t val = INREG(reg);
+	uint32_t val;
 	DipInfoFrame frame;
 	char option[32];
 	uint32_t option_val;
@@ -760,7 +844,7 @@ static void change_avi_infoframe(Transcoder transcoder, char *commands)
 static void change_spd_infoframe(Transcoder transcoder, char *commands)
 {
 	Register reg = get_dip_ctl_reg(transcoder);
-	uint32_t val = INREG(reg);
+	uint32_t val;
 	DipInfoFrame frame;
 	char option[16];
 	char option_val_s[32];
@@ -772,7 +856,7 @@ static void change_spd_infoframe(Transcoder transcoder, char *commands)
 	val = INREG(reg);
 
 	while (1) {
-		rc = sscanf(current, "%31s%n", option, &read);
+		rc = sscanf(current, "%15s%n", option, &read);
 		current = &current[read];
 		if (rc == EOF) {
 			break;
