@@ -749,6 +749,9 @@ static struct reg_debug intel_debug_regs[] = {
 /*    DEFINEREG(UDIB_SHA_BLANK_CODES), CRL only */
 	DEFINEREG(SDVOUDI),
 	DEFINEREG(DSPARB),
+	DEFINEREG(FW_BLC),
+	DEFINEREG(FW_BLC2),
+	DEFINEREG(FW_BLC_SELF),
 	DEFINEREG(DSPFW1),
 	DEFINEREG(DSPFW2),
 	DEFINEREG(DSPFW3),
@@ -2000,8 +2003,8 @@ static struct reg_debug gen6_fences[] = {
 
 static struct reg_debug ironlake_debug_regs[] = {
 	DEFINEREG(PGETBL_CTL),
-	DEFINEREG(GEN6_INSTDONE_1),
-	DEFINEREG(GEN6_INSTDONE_2),
+	DEFINEREG(INSTDONE_I965),
+	DEFINEREG(INSTDONE_1),
 	DEFINEREG2(CPU_VGACNTRL, i830_debug_vgacntrl),
 	DEFINEREG(DIGITAL_PORT_HOTPLUG_CNTRL),
 
@@ -2473,7 +2476,7 @@ static struct reg_debug i945gm_mi_regs[] = {
 	DEFINEREG(HWS_PGA),
 	DEFINEREG(IPEIR),
 	DEFINEREG(IPEHR),
-	DEFINEREG(INST_DONE),
+	DEFINEREG(INSTDONE),
 	DEFINEREG(NOP_ID),
 	DEFINEREG(HWSTAM),
 	DEFINEREG(SCPD0),
@@ -2854,12 +2857,12 @@ static uint32_t power_well_get(void)
 	if (!IS_HASWELL(devid))
 		return 0;
 
-	ret = INREG(HSW_PWR_WELL_CTL4) & HSW_PWR_WELL_ENABLE;
+	ret = INREG(HSW_PWR_WELL_CTL4) & HSW_PWR_WELL_ENABLE_REQUEST;
 
-	OUTREG(HSW_PWR_WELL_CTL4, HSW_PWR_WELL_ENABLE);
+	OUTREG(HSW_PWR_WELL_CTL4, HSW_PWR_WELL_ENABLE_REQUEST);
 
 	for (i = 0; i < 20; i++) {
-		if (INREG(HSW_PWR_WELL_CTL4) & HSW_PWR_WELL_STATE)
+		if (INREG(HSW_PWR_WELL_CTL4) & HSW_PWR_WELL_STATE_ENABLED)
 			break;
 		usleep(1000);
 	}
